#!/usr/bin/python3
__copyright__ = """
 SPDX-FileCopyrightText: Copyright (c) 2024-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 SPDX-License-Identifier: BSD-3-Clause

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 3. Neither the name of the copyright holder nor the names of its
 contributors may be used to endorse or promote products derived from
 this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

import sys
from .exceptions.exception import HandlerException
from .operations.basehandler import BaseHandler
from .operations.queryonlyhandler import QueryOnlyHandler
from .operations.clearhandler import ClearHandler
from .communication.sim.sim import SimExchangeData
from .communication.real.real import RealExchangeData
from .yamloperations.yamloperations import YAMLOperations
from .utilities.parse import parse_args
from .utilities.version import check_minimal_version, compare_string_versions

DPA_RESOURCE_TOOL_VERSION = "25.04.0169"


# Function for choose object for working mode

def choose_mode(mode):
    mode_dict = {
        "sim": SimExchangeData,
        "real": RealExchangeData
    }

    if mode in mode_dict:
        return mode_dict[mode]

    raise HandlerException(f"Mode \"{mode}\" does not supported yet")


# Function for choose object for working mode

def choose_config_handler(version):
    handler_dict = {
        "25.04": BaseHandler,
    }

    if version in handler_dict:
        return handler_dict[version]

    print(f"Version \"{version}\" is not supported yet.")
    version_list = ", ".join(list(handler_dict.keys()))
    print(f"Choose from the supported versions - {version_list}")
    raise HandlerException("Input version failure")


# Function for check minimal version for query

def check_minimal_version_query(version):
    supported_version = "25.04"
    if compare_string_versions(version, supported_version) < 0:
        print(f"Version \"{version}\" is obsolete.")
        print(f"Query is supported from {supported_version}")
        raise HandlerException("Tool version failure")


# main function

def main():
    check_minimal_version()
    try:
        options = parse_args()
        if not options.cmd:
            raise HandlerException("Command is not specified")

        if options.cmd == "config":
            chip_data = choose_mode(options.mode)
            yaml_obj = YAMLOperations()
            yaml_ver = yaml_obj.read_yaml_file(options.filename)
            handler_obj = choose_config_handler(yaml_ver)
            handler = handler_obj(parse_options=options, yaml_data=yaml_obj,
                                  exchange_obj=chip_data)
            handler.run()

        elif options.cmd == "query":
            chip_data = choose_mode(options.mode)
            chip_handler = chip_data(options.device, debug=False)
            version = chip_handler.get_tool_version()
            check_minimal_version_query(version)
            handler = QueryOnlyHandler(parse_options=options, exchange_obj=chip_data)
            handler.run()

        elif options.cmd == "clear":
            chip_data = choose_mode(options.mode)
            chip_handler = chip_data(options.device, debug=False)
            version = chip_handler.get_tool_version()
            check_minimal_version_query(version)
            handler = ClearHandler(parse_options=options, exchange_obj=chip_data)
            handler.run()

        elif options.cmd == "version":
            print(f"{DPA_RESOURCE_TOOL_VERSION}")

    except KeyboardInterrupt as k:
        sys.exit(k)
    except HandlerException as k:
        sys.exit(k)
    return 0


# main module
if __name__ == "__main__":
    sys.exit(main())
