#!/usr/bin/python3
__copyright__ = """
 SPDX-FileCopyrightText: Copyright (c) 2024-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 SPDX-License-Identifier: BSD-3-Clause

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 3. Neither the name of the copyright holder nor the names of its
 contributors may be used to endorse or promote products derived from
 this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

# Base module for check input database, input from the chip, create partitions
# and group on the chip, and create output yaml files.


import re
from ..exceptions.exception import HandlerException
from ..constants.constants import PARTITIONS, ROOT_PARTITION_GROUPS


# Main class for destroy operation
class ClearHandler:
    def __init__(self, **kwargs):
        self._options = kwargs['parse_options']
        self._chip_data = kwargs['exchange_obj'](self._options.device, debug=self._options.verbose)
        self._elements = {PARTITIONS: {},
                          ROOT_PARTITION_GROUPS: []}

    # Function to parse partition query
    def _parse_partition_query(self, text):

        out_string = "\n".join(text)
        pattern = r"(\d+)\) EU Partition ID: (\d+)\n" \
                  r"Maximal number of groups: (\d+)\n" \
                  r"The partition has a total of \d+ associated VHCA IDs, namely: [\d,-]+\n" \
                  r"Partition's member EUs are: [\d,-]+"

        partition_pattern = re.findall(pattern, out_string)

        for partition in partition_pattern:
            partition_dict = {}
            partition_dict['partition_nmb'] = int(partition[0])
            partition_dict['partition_id'] = int(partition[1])
            partition_dict['max_groups'] = int(partition[2])
            if partition_dict['max_groups']:
                out_list = self._chip_data.read_dpa_groups_query(partition_dict['partition_id'])
                partition_dict['groups'] = self._parse_group_query(out_list)
            self._elements['partitions'][int(partition[0])] = partition_dict

    # Function to parse eu group query
    def _parse_group_query(self, text, root_group=False):

        out_string = "\n".join(text)
        if root_group:
            pattern = r"(\d+)\) EU group ID: (\d+)\n" \
                      r"EU group name: \S+\n" \
                      r"Member EUs are: [\d,-]+\n"
        else:
            pattern = r"(\d+)\) EU group ID: (\d+)\n" \
                      r"EU group name: \S+\n" \
                      r"Partition ID: \S+\n" \
                      r"Member EUs are: [\d,-]+\n"

        group_list = []

        group_pattern = re.findall(pattern, out_string)

        for group in group_pattern:
            group_dict = {}
            group_dict['group_nmb'] = int(group[0])
            group_dict['group_id'] = int(group[1])
            group_list.append(group_dict)

        return group_list

    # Function to read and parse status
    def _read_and_parse_status(self):
        out_list = self._chip_data.read_dpa_partitions_query()
        self._parse_partition_query(out_list)
        out_list = self._chip_data.read_dpa_groups_query()
        self._elements[ROOT_PARTITION_GROUPS] = self._parse_group_query(out_list, root_group=True)

    # Function for destroy partitions, groups and root_partition_groups
    def _clear(self):
        passed = True
        destroy_num = 0

        for partition in self._elements[PARTITIONS].values():
            groups_destroyed = True
            if 'groups' in partition:
                for group in partition['groups']:
                    try:
                        self._chip_data.run_command("eu_group", "destroy",
                                                    id_group=group['group_id'],
                                                    id_partition=partition['partition_id'])
                        destroy_num += 1
                    except HandlerException as exc:
                        groups_destroyed = False
                        passed = False
                        print(f"Group {group['group_id']} in "
                              f"Partition {partition['partition_id']} cannot be destroyed - {exc}")

            if groups_destroyed:
                try:
                    self._chip_data.run_command("partition", "destroy",
                                                id_partition=partition['partition_id'])
                    destroy_num += 1
                except HandlerException as exc:
                    passed = False
                    print(f"Partition {partition['partition_id']} cannot be destroyed - {exc}")

        for group in self._elements[ROOT_PARTITION_GROUPS]:
            try:
                self._chip_data.run_command("eu_group", "destroy", id_group=group['group_id'])
                destroy_num += 1
            except HandlerException as exc:
                passed = False
                print(f"Group {group['group_id']} cannot be destroyed - {exc}")

        if not passed:
            raise HandlerException("Not all partitions and groups were destroyed\n"
                                   "Probably some processes running on the chip.\n"
                                   "Destroy them before running this script again.")
        if destroy_num > 0:
            print("Successfully destroyed all partitions and groups.")

    # Main function for work
    def run(self):
        self._read_and_parse_status()
        self._clear()
