#!/usr/bin/python3
__copyright__ = """
 SPDX-FileCopyrightText: Copyright (c) 2024-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 SPDX-License-Identifier: BSD-3-Clause

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 3. Neither the name of the copyright holder nor the names of its
 contributors may be used to endorse or promote products derived from
 this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

# Module that contain class SimDPAData

import sys
from ..simenums.simenums import DPAENUM, DPADB
from ..utilities.string import create_string_from_list, expand_range_from_string

# Define for VHCA
VHCA_IS_FREE = -1


# Class that contain SimDPAData - class that handle like RTOS operations
# with partitions, groups, VHCA and EU.
class SimDPAData:
    def __init__(self, arg_data):
        self._number_of_partitions = 0
        self._number_of_groups = 1
        self._chip_data = arg_data
        self._data = {}
        self._init()
        self._fill()

    def get_data(self):
        return self._data

    def _init(self):
        # Components of EU - partition id, relative index, group number
        try:
            self._data['EU'] = [[0, index, 0]
                                for index in range(self._chip_data[DPADB.CHIP]['max_num_dpa_eu'])]
        except KeyError:
            self._error_message("DPA database file empty or corrupted.")
        self._data['VHCA'] = {}
        for vhca in self._chip_data[DPADB.VHCA]:
            self._data['VHCA'][vhca['vhca_id']] = VHCA_IS_FREE
        self._data['Partitions'] = {}
        self._data['Groups'] = []

    def _error_message(self, message):
        print(message)
        sys.exit(1)

    def fill_group(self, partition_id, group):
        chip_data = self._chip_data[DPADB.CHIP]
        if partition_id == 0:
            if self._number_of_groups + 1 > chip_data['max_eu_groups']:
                self._error_message(f"Number of the groups {self._number_of_groups} exceeded "
                                    f"upper limit of groups {chip_data['max_eu_groups']}")
            self._number_of_groups += 1

        eu_list = expand_range_from_string(group['range'])
        if eu_list[-1] >= self._chip_data[DPADB.CHIP]['max_num_dpa_eu']:
            self._error_message("Size of the group exceeded upper limit of partition")
        for group_eu in eu_list:
            found = False
            for sub_eu, eu in enumerate(self._data['EU']):
                if eu[DPAENUM.PARTITION] == partition_id and eu[DPAENUM.REL_INDEX] == group_eu:
                    if eu[DPAENUM.GROUP]:
                        self._error_message("There is an overlap of EUs between the mask given "
                                            "and another existing EU object (group/partition)")
                    else:
                        self._data['EU'][sub_eu][DPAENUM.GROUP] = group['id']
                        found = True
            if not found:
                self._error_message("The size of group is over than partition size")
        if not partition_id:
            self._data['Groups'].append(group)
        else:
            if 'groups' not in self._data['Partitions'][partition_id]:
                self._data['Partitions'][partition_id]['groups'] = []
            self._data['Partitions'][partition_id]['groups'].append(group)

    def read_group(self, partition_id, group_id):
        if not partition_id:
            group_list = self._data['Groups']
        else:
            group_list = self._data['Partitions'][partition_id]['groups']
        if not group_list or len(group_list) == 0:
            return None
        group = {}

        for group_value in group_list:
            if group_value['id'] == group_id:
                group['id'] = group_id
                eu_range = [value[DPAENUM.REL_INDEX] for value in self._data['EU']
                            if value[DPAENUM.PARTITION] == partition_id
                            and value[DPAENUM.GROUP] == group_id]
                group['range'] = create_string_from_list(eu_range)
                group['name'] = group_value['name']
                return group

        return None

    def fill_partition(self, partition):
        chip_data = self._chip_data[DPADB.CHIP]
        if self._number_of_partitions + 1 > chip_data['max_num_dpa_eu_partitions']:
            self._error_message("Number of the partition exceeded upper limit of partition")
        self._number_of_partitions += 1
        max_groups = partition['max_num_eu_group']
        if self._number_of_groups + max_groups + 1 > chip_data['max_eu_groups']:
            self._error_message(f"Number of the groups {self._number_of_groups + max_groups + 1} "
                                f"exceeded upper limit of groups {chip_data['max_eu_groups']}")
        self._number_of_groups += max_groups + 1
        ident = partition['id']
        if ident in self._data['Partitions']:
            self._error_message(f"Partition Id {ident} is already exist")
        self._data['Partitions'][ident] = {}
        self._data['Partitions'][ident]['max_group'] = partition['max_num_eu_group']
        vhca_list = expand_range_from_string(partition['vhca_list'])
        for vhca in vhca_list:
            if vhca not in self._data['VHCA'].keys():
                self._error_message("Error: this VHCA's partition doesn't and can't contain "
                                    "any partition")
            if self._data['VHCA'][vhca] != VHCA_IS_FREE:
                self._error_message("There is an overlap of VHCA IDs between the VHCA IDs "
                                    "given and another existing EU partition. A VHCA's ID "
                                    "can belong at most to a single partition")
            for vhca_elem in self._chip_data[DPADB.VHCA]:
                if vhca_elem['vhca_id'] == vhca:
                    if 'parent_vhca_id' not in vhca_elem:
                        self._error_message(f"Error: this vhca_id {vhca} is root and doesn't "
                                            "and can't contain any partition")
                    function_type = vhca_elem['function_type']
                    if function_type == 1:
                        self._error_message(f"Error: this vhca_id {vhca} is VF and doesn't "
                                            "and can't contain any partition")
                    if function_type == 2:
                        self._error_message(f"Error: this vhca_id {vhca} is SF and doesn't "
                                            "and can't contain any partition")
            self._data['VHCA'][vhca] = ident
        eu_list = expand_range_from_string(partition['range'])
        if eu_list[-1] >= self._chip_data[DPADB.CHIP]['max_num_dpa_eu']:
            self._error_message("Size of the group exceeded upper limit of partition")
        for sub_eu, eu in enumerate(eu_list):
            if self._data['EU'][eu][DPAENUM.PARTITION] or self._data['EU'][eu][DPAENUM.GROUP]:
                self._error_message("There is an overlap of EUs between the mask given and "
                                    "another existing EU object (group/partition)")
            self._data['EU'][eu][DPAENUM.PARTITION] = ident
            self._data['EU'][eu][DPAENUM.REL_INDEX] = sub_eu
        if 'groups' in partition.keys():
            for group in partition['groups']:
                self.fill_group(ident, group)

    def read_partition(self, ident):
        partition = {}
        if ident not in self._data['Partitions'].keys():
            return None
        partition['id'] = ident
        partition['max_num_eu_group'] = self._data['Partitions'][ident]['max_group']
        vhca_list = [vhca_id for vhca_id in self._data['VHCA']
                     if self._data['VHCA'][vhca_id] == ident]
        partition['vhca_list'] = create_string_from_list(vhca_list)
        eu_list = [eu for eu, value in enumerate(self._data['EU'])
                   if value[DPAENUM.PARTITION] == ident]
        partition['range'] = create_string_from_list(eu_list)
        return partition

    def _fill(self):
        if self._chip_data[DPADB.PARTITIONS] and len(self._chip_data[DPADB.PARTITIONS]) > 0:
            for partition in self._chip_data[DPADB.PARTITIONS]:
                self.fill_partition(partition)
        if self._chip_data[DPADB.ROOT_GROUPS] and len(self._chip_data[DPADB.ROOT_GROUPS]) > 0:
            for group in self._chip_data[DPADB.ROOT_GROUPS]:
                self.fill_group(0, group)
