#!/usr/bin/python3
__copyright__ = """
 SPDX-FileCopyrightText: Copyright (c) 2024-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 SPDX-License-Identifier: BSD-3-Clause

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 3. Neither the name of the copyright holder nor the names of its
 contributors may be used to endorse or promote products derived from
 this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

# Module that contain class ExtCommandHandler

from ..operation import OperationHandler
from ...simenums.simenums import DPADB
from ...simdatabases.simextdb import DPADBInit, SimDataManager


# Function to convert PCI BDF to hex string

def convert_pci_bdf_hex(fw):
    return f"{fw >> 8:02x}:{(fw >> 3) & 0x1f:02x}.{fw & 0x7:x}"


# Class for execution of ext_command subcommands that
# not a part and should be never be a part of dpaeumgmt.
# The commands should be used by user, outside of dpa-resource-mgmt tool.
# set_processes - change the number of the processes for test info status handling.
# reset - same as chip reset, remove all partitions, groups and processes.
# init - used if database file for device is absent and create new file
# with default chip and vhca parameters.
class ExtCommandHandler(OperationHandler):
    def __init__(self, arg_params, filename):
        self._read_file = bool(arg_params.subcommand != "init")
        super().__init__(arg_params, filename)

    def _get_subfunction(self):
        subfunc_dict = {"set_processes": self.__set_processes,
                        "reset": self.__reset,
                        "init": self.__init}
        return subfunc_dict[self._params.subcommand]

    def _is_read_cfg_file(self):
        return self._read_file

    def __set_processes(self):
        self._data[DPADB.PROCESSES] = DPADBInit().init(DPADB.PROCESSES)
        self._data[DPADB.PROCESSES]['processes'] = self._params.processes_num
        self._set_update()

    def __reset(self):
        self._data[DPADB.PARTITIONS] = DPADBInit().init(DPADB.PARTITIONS)
        self._data[DPADB.ROOT_GROUPS] = DPADBInit().init(DPADB.ROOT_GROUPS)
        self._data[DPADB.PROCESSES] = DPADBInit().init(DPADB.PROCESSES)
        self._set_update()

    def __init(self):
        self._data = SimDataManager().data
        chip = {
            "max_processes": 64,
            "max_process_per_vhca": 64,
            "max_threads": 40000,
            "max_threads_per_process": 20000,
            "thread_stack_size": 16384,
            "max_eu_groups": 16,
            "heap_max_size": 0x40000000,
            "num_cores": 16}

        chip['cores'] = []
        eu_id = 0
        core_id = 0
        active_cores = 0

        for core_index in range(chip['num_cores']):
            # Skip cores 10-13 for progcc
            if 10 <= core_index <= 13:
                core_id += 1
                continue
            core_elem = {'core_id': core_id, 'eu_num': 16}
            # Set EU number to 14 for last core (skip 2 EUs for kernel)
            if core_index == chip['num_cores'] - 1:
                core_elem['eu_num'] = 14
            core_elem['eus'] = []
            for eu_index in range(core_elem['eu_num']):
                eu_elem = {'core_id': core_id, 'eu_index': eu_index, 'eu_id': eu_id}
                core_elem['eus'].append(eu_elem)
                eu_id += 1
            chip['cores'].append(core_elem)
            core_id += 1
            active_cores += 1

        chip['num_cores'] = active_cores
        chip["max_num_dpa_eu_partitions"] = 16
        chip["max_num_dpa_eu_per_group"] = eu_id
        chip["max_num_partition_vhca_id"] = 16
        chip["max_num_dpa_eu"] = eu_id

        self._data[DPADB.CHIP] = chip

        vhca = []

        pci_bdf = 0x8000
        vhca_first_elem = {"vhca_id": 1, "function_type": 3,
                           "pci_bdf": convert_pci_bdf_hex(pci_bdf), "vport_id": 0}
        vhca.append(vhca_first_elem)
        for index in range(19):
            pci_bdf += 1
            vhca_elem = {"vhca_id": index + 2, "function_type": 0,
                         "parent_vhca_id": 1, "parent_function_type": 3,
                         "pci_bdf": convert_pci_bdf_hex(pci_bdf), "vport_id": 0}
            vhca.append(vhca_elem)
        self._data[DPADB.VHCA] = vhca

        self._set_update()
