#!/usr/bin/python3
__copyright__ = """
 SPDX-FileCopyrightText: Copyright (c) 2024-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 SPDX-License-Identifier: BSD-3-Clause

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 3. Neither the name of the copyright holder nor the names of its
 contributors may be used to endorse or promote products derived from
 this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

import argparse
import re


# Function to validate device name
def validate_device_name(value):
    if re.fullmatch(r'[A-Za-z0-9\._/]+', value):
        return value
    raise argparse.ArgumentTypeError(
        "Only letters, digits, underscores (_) and slashes (/), and dot (.) are allowed."
    )


# Function to parse arguments
def parse_args():
    modes = ['sim', 'real', 'mix']
    default_mode = modes[1]

    common_args = argparse.ArgumentParser(add_help=False)
    common_args.add_argument('-d', '--device', dest='device', help="ibv device name.",
                             type=validate_device_name, required=True)
    common_args.add_argument('-m', '--mode', dest='mode', choices=modes,
                             help=argparse.SUPPRESS, default=default_mode)
    common_args.add_argument('-v', '--verbose', dest='verbose',
                             help="show commands of the tool.", action="store_true")

    parser = argparse.ArgumentParser(add_help=True)
    sub_parser = parser.add_subparsers(dest='cmd')
    config = sub_parser.add_parser("config", parents=[common_args],
                                   description="Config chip with configuration file")
    config.add_argument('-f', '--filename', dest='filename',
                        help="name of configuration YAML file.", required=True)
    config.add_argument('-o', '--output', dest='output',
                        help="the template for output files.", default=".")
    query = sub_parser.add_parser("query", parents=[common_args],
                                  description="Only query device info")
    query.add_argument('-t', '--type', dest='type', choices=['chip', 'vhca', 'resources'],
                       help="output data type.", required=True)
    query.add_argument('-f', '--filename', dest='filename',
                       help="Output filename. By default is stdout.")
    query.add_argument('-y', '--yaml-format', dest='yaml_format',
                       help="Output format. By default - human-readable",
                       action="store_true")
    sub_parser.add_parser("clear", parents=[common_args],
                          description="Clear previously allocated configuration")
    sub_parser.add_parser("version", description="Version of the tool")
    return parser.parse_args()
