#!/usr/bin/python3
__copyright__ = """
 SPDX-FileCopyrightText: Copyright (c) 2024-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 SPDX-License-Identifier: BSD-3-Clause

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 3. Neither the name of the copyright holder nor the names of its
 contributors may be used to endorse or promote products derived from
 this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

from ..exceptions.exception import HandlerException

INDEX_INVALID_VALUE = -2


# expand range from free values
def expand_range_from_string(string, suppress_value_error_exc=True):
    result = []
    for part in string.replace(" ", "").split(','):
        try:
            if '-' in part:
                start, end = map(int, part.split('-'))
                result.extend(range(start, end + 1))
            else:
                result.append(int(part))
        except ValueError as exc:
            if not suppress_value_error_exc:
                raise
            raise HandlerException(f"string {string} contain wrong value - {exc}") from exc
        except MemoryError as exc:
            raise HandlerException(f"string {string} contain too big value as {exc}") from exc
    return result


# Function for return correct substring from first and last values
def create_substring_in_range(first, last):
    if last - first > 1:
        sub_string = f"{first}-{last}"
    elif last - first == 1:
        sub_string = f"{first},{last}"
    else:
        sub_string = f"{first}"
    return sub_string


# Function for return normalized string
def create_string_from_dict(input_dict, value=True):
    eu_list = []
    first = last = curr = INDEX_INVALID_VALUE
    for idx, elem in input_dict.items():
        curr = elem if value else idx
        if first == INDEX_INVALID_VALUE:
            first = curr
        elif last + 1 < curr:
            if first != INDEX_INVALID_VALUE:
                eu_list.append(create_substring_in_range(first, last))
            first = curr
        last = curr
    if first != INDEX_INVALID_VALUE:
        eu_list.append(create_substring_in_range(first, last))
        first = INDEX_INVALID_VALUE

    return ",".join(eu_list)


# Function for return normalized string
def create_string_from_list(input_list):
    elem_list = []
    first = last = curr = INDEX_INVALID_VALUE
    for elem in input_list:
        curr = int(elem)
        if first == INDEX_INVALID_VALUE:
            first = curr
        elif last + 1 < curr:
            if first != INDEX_INVALID_VALUE:
                elem_list.append(create_substring_in_range(first, last))
            first = curr
        last = curr
    if first != INDEX_INVALID_VALUE:
        elem_list.append(create_substring_in_range(first, last))
        first = INDEX_INVALID_VALUE
    return ",".join(elem_list)


# Function for convert strings to list of strings
def convert_values_to_list(value, main_key, key):
    try:
        if isinstance(value, (dict, str)):
            allocation = [value]
        elif isinstance(value, int):
            allocation = [str(value)]
        elif isinstance(value, list):
            allocation = [str(val) if isinstance(val, int)
                          else val for val in value]
        else:
            raise HandlerException(f"Unknown type {key} of {value} in {main_key}")
    except Exception as exc:
        raise HandlerException(f"Invalid type {key} of {value} in {main_key} - {exc}") from exc
    return allocation


# Function for get value from dictionary and convert it to list
def get_value_as_list(dict_db, partition_key, key, raise_exc=True):
    try:
        temp_allocation = dict_db[partition_key][key]
    except KeyError as exc:
        if not raise_exc:
            return None
        raise HandlerException(f"Missing {key} in partition {partition_key}") from exc
    return convert_values_to_list(temp_allocation, partition_key, key)


# Function for find string in the list of the strings by keyword list
# and return value.
# For example:
# From "EU group ID: 2" by keywords "group" and "ID" return 2
def get_value_from_list_of_strings(strings_list, keyword_list):
    for string in strings_list:
        if all(element in string for element in keyword_list):
            return string.split(":")[1].strip()

    return None
